/*
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2, or (at
  your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  Copyright (c) Mat Kovach <mkovach@alal.com> 2003,2004
*/

/*
   nspam.c --
        An AOLserver module to allow pam authentication
 */

/*
 *
 * $Log: nspam.c,v $
 * Revision 1.1.1.1  2005/02/15 19:38:33  mek
 * initial import
 *
 * Revision 1.3  2004/07/15 12:17:59  kovachme
 * updating years
 *
 * Revision 1.2  2004/07/15 12:11:42  kovachme
 * updating years
 *
 * Revision 1.1.1.1  2004/06/02 14:27:19  kovachme
 * initial import
 *
 * Revision 1.2  2004/06/02 14:20:32  kovachme
 * updating error codes
 *
 * Revision 1.1.1.1  2004/06/02 14:02:47  kovachme
 * initial import
 *
 * Revision 1.3  2004/04/27 14:18:50  kovachme
 * fixes
 *
 *
 */

#include "ns.h"
#include "pam_support.h"

#ifdef MACOSX
#include <pam_appl.h>
#else
#include <security/pam_appl.h>
#endif

#define DEFAULT_PAM_DOMAIN	"aolserver"
#define PAM_CONFIG_PREFIX	"PamDomain"

/*
 *  structures
 */

typedef struct {
    char *hServer;
    char *hModule;
    char *PamDomain;
} PamContext;

NS_EXPORT int Ns_ModuleVersion = 1;

/*
 * Private functions
 */

NS_EXPORT int Ns_ModuleInit();
NS_EXPORT int NsPamInterpInit();
NS_EXPORT int PamCmd();

NS_EXPORT int Ns_ModuleInit(hServer, hModule)
     char *hServer;
     char *hModule;
{
  char *ConfigPath;
  char *PamDomain;
  PamContext *ctx;

  ctx = Ns_Malloc(sizeof(PamContext));
  ctx->hServer = hServer;
  ctx->hModule = hModule;
  
  if ((ConfigPath = Ns_ConfigGetPath(hServer, hModule, NULL)) == NULL) {
    Ns_Log(Warning, "Module Init for %s/%s: no section found", hServer,
	   hModule);
  }
  
  if ((PamDomain =
       Ns_ConfigGetValue(ConfigPath, PAM_CONFIG_PREFIX)) == NULL) {
    Ns_Log(Warning,
	   "Module Init for %s/%s: using default PamDomain of %s",
	   hServer, hModule, DEFAULT_PAM_DOMAIN);
    PamDomain = DEFAULT_PAM_DOMAIN;
  }
  
  ctx->PamDomain = PamDomain;
  Ns_Log(Notice, "Module Init for %s/%s: PamDomain: %s", ctx->hServer,
	 hModule, ctx->PamDomain);
  return (Ns_TclInitInterps(hServer, NsPamInterpInit, ctx));
  
}

NS_EXPORT int NsPamInterpInit(interp, context)
     Tcl_Interp *interp;
     void *context;
{
  
  Tcl_CreateCommand(interp, "ns_pam", PamCmd, context, NULL);
  return NS_OK;
}

NS_EXPORT int PamCmd(context, interp, argc, argv)
     ClientData context;
     Tcl_Interp *interp;
     int argc;
     char **argv;
{
  int rc;
  PamContext *ctx = context;
  
  if (argc == 4) {
    if (strcmp(argv[1], "auth") == 0) {
      rc = auth_pam(argv[2], argv[3], ctx->PamDomain, NULL);
      switch (rc) {
      case PAM_SUCCESS:
	Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
	return NS_OK;
	break;
      case PAM_AUTH_ERR:
	Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
	return NS_OK;
	break;
      default:
	Tcl_AppendResult(interp, pam_strerror(NULL, rc), NULL);
	return NS_ERROR;
      }
    } else {
      Tcl_AppendResult(interp, "Illegal Command: auth passwd chpass are valid commands", NULL);
      return NS_ERROR;
    }
  }
  
  if (argc == 5) {
    if (strcmp(argv[1], "chpass") == 0) {
      rc = auth_pam(argv[2], argv[3], ctx->PamDomain, argv[4]);
      switch (rc) {
      case PAM_SUCCESS:
	Tcl_SetObjResult(interp, Tcl_NewIntObj(1));
	return NS_OK;
	break;
      case PAM_AUTH_ERR:
	Tcl_SetObjResult(interp, Tcl_NewIntObj(0));
	return NS_OK;
	break;
      default:
	Tcl_AppendResult(interp, pam_strerror(NULL, rc), NULL);
	return NS_ERROR;
      }
    } else {
      Tcl_AppendResult(interp, "Illegal Command: auth passwd chpass are valid commands", NULL);
      return NS_ERROR;
    }
  }
  
  Tcl_AppendResult(interp, "Illegal usage", NULL);
  
  return NS_ERROR;
}
