/*
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2, or (at
  your option) any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  Copyright (c) Mat Kovach <mkovach@alal.com> 2003,2004
*/

/*
 *
 * $Log: pam_support.c,v $
 * Revision 1.1.1.1  2005/02/15 19:38:33  mek
 * initial import
 *
 * Revision 1.2  2004/07/15 12:11:42  kovachme
 * updating years
 *
 * Revision 1.1.1.1  2004/06/02 14:27:19  kovachme
 * initial import
 *
 * Revision 1.2  2004/06/02 14:10:17  kovachme
 * updating pam support routintes
 *
 * Revision 1.4  2004/05/19 14:55:43  kovachme
 * Minor typo in code
 *
 * Revision 1.3  2004/04/27 14:18:50  kovachme
 * fixes
 *
 * 
 */

#include "pam_support.h"
#include <ns.h>

#ifdef MACOSX
#include <pam_appl.h>
#else
#include <security/pam_appl.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define COPY_STRING(s) (s) ? strdup(s) : NULL

struct commands {
  int (*func) (pam_handle_t *, int);
  int arg;
};

typedef struct {
  char *user;
  char *pass;
  char *new_pass;
  int  given_current_password_p;
} pam_info;

struct commands auth_commands[] = {
  {pam_authenticate, 0}
  , {pam_acct_mgmt, 0}
  , {NULL, 0}
};

struct commands open_session_commands[] = {
  {pam_setcred, PAM_ESTABLISH_CRED}
  , {pam_open_session, PAM_SILENT}
  , {NULL, 0}
};

struct commands close_session_commands[] = {
  {pam_close_session, PAM_SILENT}
  , {NULL, 0}
};

struct commands change_password_commands[] = {
  {pam_chauthtok, PAM_SILENT}
  , {NULL, 0}
};

static int auth_pam_conv(num_msg, msg, resp, appdata_ptr)
     int num_msg;
     const struct pam_message **msg;
     struct pam_response **resp;
     void *appdata_ptr;
{
  int i = 0;
  struct pam_response *reply = NULL;
  pam_info *info = appdata_ptr;

  reply = malloc(sizeof(struct pam_response) * num_msg);

  if (!reply)
    return PAM_CONV_ERR;

  for (i = 0; i < num_msg; i++) {
    switch (msg[i]->msg_style) {
    case PAM_PROMPT_ECHO_ON:
      reply[i].resp_retcode = PAM_SUCCESS;
      reply[i].resp = COPY_STRING(info->user);
      break;
    case PAM_PROMPT_ECHO_OFF:

      if (info->given_current_password_p==0) 
      { 
        reply[i].resp = COPY_STRING(info->pass);
        info->given_current_password_p = 1;
      } else {
        reply[i].resp = COPY_STRING(info->new_pass);
      }

      if (reply[i].resp == NULL)
	reply[i].resp_retcode = PAM_CONV_ERR;

      reply[i].resp_retcode = PAM_SUCCESS;
      break;
    case PAM_TEXT_INFO:
      /* fall through */
    case PAM_ERROR_MSG:
      reply[i].resp_retcode = PAM_SUCCESS;
      reply[i].resp = NULL;
      break;
    default:
      /* Must be an error of some sort... */
      free(reply);
      return PAM_CONV_ERR;
    }
  }

  *resp = reply;
  return PAM_SUCCESS;
}

static int auth_pam_end(pamh, pam_error)
     pam_handle_t *pamh;
     int pam_error;
{
  if (pam_end(pamh, pam_error) != PAM_SUCCESS) {
    pamh = NULL;
    return -1;
  }
  return pam_error;
}

static int auth_pam_check(pamh, pam_error)
     pam_handle_t *pamh;
     int pam_error;
{
  if (pam_error != PAM_SUCCESS)
    pam_error = auth_pam_end(pamh, pam_error);
  return pam_error;
}

int auth_pam(username, password, domain, newpassword)
     char *username;
     char *password;
     char *domain;
     char *newpassword;
{
  pam_handle_t *pamh;
  int pam_error;
  pam_info info;
  struct pam_conv conv;
  struct commands *c = auth_commands;;

  if (!(username && password && strlen(username) && strlen(password)))
    return -1;

  if (!(domain && strlen(domain)))
    return -1;

  info.pass = password;
  info.user = username;
  info.new_pass = NULL;
  info.given_current_password_p=0;

  conv.conv = auth_pam_conv;
  conv.appdata_ptr = &info;

  pam_error = pam_start(domain, username, &conv, &pamh);
  if (auth_pam_check(pamh, pam_error) != PAM_SUCCESS)
    return pam_error;

  /* We started pam handleer, now run the functions */
  while (c->func != NULL) {
    pam_error = c->func(pamh, c->arg);
    if (auth_pam_check(pamh, pam_error) != PAM_SUCCESS)
      return pam_error;
    c++;
  }

  if ((newpassword && strlen(newpassword))) {

    info.new_pass = newpassword;
    info.given_current_password_p=0;

    c = change_password_commands;
    while (c->func != NULL) {
      pam_error = c->func(pamh, c->arg);
      if (auth_pam_check(pamh, pam_error) != PAM_SUCCESS)
	return pam_error;
      c++;
    }
  }

  return auth_pam_end(pamh, pam_error);

}
